==> lvm-0.3alpha.tar.gz: LVM/0.3alpha/tools/lvmchange.c <==
/*
 * Author: Heinz Mauelshagen, Germany
 *
 * March-May 1997
 *
 * LVM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * LVM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA. 
 *
 */

#include <lvm_user.h>

#ifdef DEBUG
int opt_d = 0;
#endif

int main ( int argc, char **argv) {
   int c = 0;
   int c1 = 0;
   int lvm = -1;
   int opt_f = 0;
   int opt_r = 0;
   int opt_R = 0;
   int opt_v = 0;
   int read_ahead = 0;
   int ret = 0;
   char *cmd = NULL;
#ifdef DEBUG
   char *options = "dfh?r:Rv";
#else
   char *options = "fh?r:Rv";
#endif

   cmd = basename ( argv[0]);

   SUSER_CHECK;

   while ( ( c = getopt ( argc, argv, options)) != EOF) {
      switch ( c) {
#ifdef DEBUG
         case 'd':
            if ( opt_d > 0) {
               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
               return 1;
            }
            opt_d++;
            break;
#endif

         case 'f':
            if ( opt_f > 0) {
               fprintf ( stderr, "%s -- f option yet given\n\n", cmd);
               return 1;
            }
            opt_f++;
            break;

         case 'h':
         case '?':
            printf ( "\n%s\n\n%s -- Logical Volume Manager change\n\n"
                     "Synopsis:\n"
                     "---------\n\n"
                     "%s\n"
#ifdef DEBUG
                     "\t[-d]\n"
#endif
                     "\t[-f]\n"
                     "\t[-h/-?]\n"
                     "\t[-r ReadAheadSectors]\n"
                     "\t[-R]\n"
                     "\t[-v]\n\n",
                     lvm_version, cmd, cmd);
            return 0;
            break;

         case 'r':
            if ( opt_r > 0) {
               fprintf ( stderr, "%s -- r option yet given\n\n", cmd);
               return 1;
            }
            opt_r++;
            read_ahead = atoi ( optarg);
            if ( read_ahead < LVM_MIN_READ_AHEAD -10||
                 read_ahead > LVM_MAX_READ_AHEAD) {
               fprintf ( stderr, "%s -- r option argument \"%s\" invalid\n\n",
                         cmd, optarg);
               return 1;
            }
            break;

         case 'R':
            if ( opt_R > 0) {
               fprintf ( stderr, "%s -- R option yet given\n\n", cmd);
               return 1;
            }
            opt_R++;
            break;

         case 'v':
            if ( opt_v > 0) {
               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
               return 1;
            }
            opt_v++;
            break;

         default:
            fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                      cmd, c);
            return 1;
      }
   }
  
   if ( opt_r == 0 && opt_R == 0) {
      fprintf ( stderr, "%s -- please give the r and/or R option\n\n", cmd);
      return 1;
   }

   if ( optind < argc) {
      fprintf ( stderr, "%s -- error on command line\n\n", cmd);
      return 1;
   }

   LVM_LOCK ( 0);
   LVM_CHECK_IOP;
   lvm_dont_interrupt ( 0);

   if ( ( lvm = open ( LVM_DEV, O_RDONLY)) == -1) {
      fprintf ( stderr, "%s -- ERROR opening %s\n\n", cmd, LVM_DEV);
      return 1;
   }

   if ( opt_r > 0) {
      if ( opt_v > 0) printf ( "%s -- changing global read ahead "
                               " for LVM to %d sectors\n",
                               cmd, read_ahead);
      if ( ( ret = ioctl ( lvm, BLKRASET, read_ahead)) < 0) {
         fprintf ( stderr, "%s -- ERROR %d doing BLKRASET ioctl\n", cmd, ret);
         ret = -2;
      } else {
         printf ( "%s -- LVM global read ahead sectors set\n", cmd);
      }
   }

   if ( opt_R > 0) {
      if ( opt_f == 0) {
         c = 0;
         while ( c != 'y' && c != 'n') {
            if ( c == '\n' || c == 0)
               printf ( "%s -- do you want to reset LVM [y/n]? ", cmd);
            c = tolower ( getchar ());
         }
      } else c = 'y';
      c1 = c;
      while ( c != '\n') c = tolower ( getchar ());
      if ( c1 == 'n') {
         printf ( "%s -- NOT resetting LVM\n\n", cmd);
         ret = 0;
      } else {
         if ( opt_v > 0) printf ( "%s -- resetting LVM ...\n", cmd);
         if ( ( ret = ioctl ( lvm, LVM_RESET, &cmd)) < 0) {
            fprintf ( stderr, "%s -- ERROR %d doing reset ioctl\n", cmd, ret);
            ret = -3;
         } else {
            printf ( "%s -- LVM has been reset\n", cmd);
         }
      }
   }

   lvm_interrupt ();
   LVM_UNLOCK ( 0);
   close ( lvm);

   printf ( "\n");
   return ret;
}

==> lvm-0.4alpha.tar.gz: LVM/0.4alpha/tools/lvmchange.c <==
/*
 * Author: Heinz Mauelshagen, Germany
 *
 * March-May 1997
 * June 1998
 *
 * LVM is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * LVM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA. 
 *
 */

/*
 * Changelog
 *
 *    06/12/1998 - enhanced checking numbers in option arguments
 *
 */

#include <lvm_user.h>

char *cmd = NULL;

#ifdef DEBUG
int opt_d = 0;
#endif

int main ( int argc, char **argv) {
   int c = 0;
   int c1 = 0;
   int lvm = -1;
   int opt_f = 0;
   int opt_r = 0;
   int opt_R = 0;
   int opt_v = 0;
   int read_ahead = 0;
   int ret = 0;
#ifdef DEBUG
   char *options = "dfh?r:Rv";
#else
   char *options = "fh?r:Rv";
#endif

   cmd = basename ( argv[0]);

   SUSER_CHECK;

   while ( ( c = getopt ( argc, argv, options)) != EOF) {
      switch ( c) {
#ifdef DEBUG
         case 'd':
            if ( opt_d > 0) {
               fprintf ( stderr, "%s -- d option yet given\n\n", cmd);
               return 1;
            }
            opt_d++;
            break;
#endif

         case 'f':
            if ( opt_f > 0) {
               fprintf ( stderr, "%s -- f option yet given\n\n", cmd);
               return 1;
            }
            opt_f++;
            break;

         case 'h':
         case '?':
            printf ( "\n%s\n\n%s -- Logical Volume Manager Change\n\n"
                     "Synopsis:\n"
                     "---------\n\n"
                     "%s\n"
#ifdef DEBUG
                     "\t[-d]\n"
#endif
                     "\t[-f]\n"
                     "\t[-h/-?]\n"
                     "\t[-r ReadAheadSectors]\n"
                     "\t[-R]\n"
                     "\t[-v]\n\n",
                     lvm_version, cmd, cmd);
            return 0;
            break;

         case 'r':
            if ( opt_r > 0) {
               fprintf ( stderr, "%s -- r option yet given\n\n", cmd);
               return 1;
            }

            if ( lvm_check_number ( optarg) == TRUE) {
               read_ahead = atoi ( optarg);
               opt_r++;
               if ( read_ahead < LVM_MIN_READ_AHEAD -10||
                    read_ahead > LVM_MAX_READ_AHEAD) {
                  fprintf ( stderr, "%s -- read ahead sector number %d is "
                                    "invalid\n\n", cmd, read_ahead);
                  return 1;
               }
            } else {
               fprintf ( stderr, "%s -- ERROR option r argument \"%s\"\n\n",
                                 cmd, optarg);
               return 1;
            }
            break;

         case 'R':
            if ( opt_R > 0) {
               fprintf ( stderr, "%s -- R option yet given\n\n", cmd);
               return 1;
            }
            opt_R++;
            break;

         case 'v':
            if ( opt_v > 0) {
               fprintf ( stderr, "%s -- v option yet given\n\n", cmd);
               return 1;
            }
            opt_v++;
            break;

         default:
            fprintf ( stderr, "%s -- invalid command line option \"%c\"\n\n",
                      cmd, c);
            return 1;
      }
   }
  
   if ( opt_r == 0 && opt_R == 0) {
      fprintf ( stderr, "%s -- please give the r and/or R option\n\n", cmd);
      return 1;
   }

   if ( optind < argc) {
      fprintf ( stderr, "%s -- error on command line\n\n", cmd);
      return 1;
   }

   LVM_LOCK ( 0);
   LVM_CHECK_IOP;
   lvm_dont_interrupt ( 0);

   if ( ( lvm = open ( LVM_DEV, O_RDONLY)) == -1) {
      fprintf ( stderr, "%s -- ERROR opening %s\n\n", cmd, LVM_DEV);
      return 1;
   }

   if ( opt_r > 0) {
      if ( opt_v > 0) printf ( "%s -- changing global read ahead "
                               " for LVM to %d sectors\n",
                               cmd, read_ahead);
      if ( ( ret = ioctl ( lvm, BLKRASET, read_ahead)) < 0) {
         fprintf ( stderr, "%s -- ERROR %d doing BLKRASET ioctl\n", cmd, ret);
         ret = -2;
      } else {
         printf ( "%s -- LVM global read ahead sectors set\n", cmd);
      }
   }

   if ( opt_R > 0) {
      if ( opt_f == 0) {
         c = 0;
         while ( c != 'y' && c != 'n') {
            if ( c == '\n' || c == 0)
               printf ( "%s -- do you want to reset LVM [y/n]? ", cmd);
            c = tolower ( getchar ());
         }
      } else c = 'y';
      c1 = c;
      while ( c != '\n') c = tolower ( getchar ());
      if ( c1 == 'n') {
         printf ( "%s -- NOT resetting LVM\n\n", cmd);
         ret = 0;
      } else {
         if ( opt_v > 0) printf ( "%s -- resetting LVM ...\n", cmd);
         if ( ( ret = ioctl ( lvm, LVM_RESET, &cmd)) < 0) {
            fprintf ( stderr, "%s -- ERROR %d doing reset ioctl\n", cmd, ret);
            ret = -3;
         } else {
            printf ( "%s -- LVM has been reset\n", cmd);
         }
      }
   }

   lvm_interrupt ();
   LVM_UNLOCK ( 0);
   close ( lvm);

   printf ( "\n");
   return ret;
}
